package org.expeditee.gio.swing;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.swing.TransferHandler;

import org.expeditee.Util;
import org.expeditee.core.Point;
import org.expeditee.gio.DragAndDropManager;
import org.expeditee.gui.DisplayController;
import org.expeditee.gui.MessageBay;

public class SwingDragAndDropManager extends DragAndDropManager {

	/** Singleton instance. */
	private static SwingDragAndDropManager _instance = null;
	
	/** Singleton instantiator. */
	public static SwingDragAndDropManager getInstance()
	{
		if (_instance == null) _instance = new SwingDragAndDropManager();
		
		return _instance;
	}

	// GNOME and KDE desktops have a specialized way of DNDing files
	private DataFlavor _URIListDataflavorString;

	private DataFlavor _URIListDataflavorCharArray;
	
	private ExpediteeTransferHandler _transferHandler;
	
	private SwingDragAndDropManager()
	{
		// Wire up the transfer handler
		if (Util.isMinimumVersion6()) {
			_transferHandler = new ExpediteeTransferHandler();
			SwingMiscManager.getIfUsingSwingGraphicsManager().setTransferHandler(_transferHandler);
		} else {
			System.err.println("Upgrade to a (minimum) of Java 1.6 to enable drag and drop support in Expeditee");
		}
		
		try {
			_URIListDataflavorString = new DataFlavor("text/uri-list;class=java.lang.String");
			_URIListDataflavorCharArray = new DataFlavor("text/uri-list;class=\"[C\"");
			
		// This would never happen, java.lang.String is always present
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			_URIListDataflavorString = null;
		}
		assert (_URIListDataflavorString != null);
		assert (_URIListDataflavorCharArray != null);
	}
	
	private class ExpediteeTransferHandler extends TransferHandler {
		
		/** Auto-generated serialisation ID#. */
		private static final long serialVersionUID = 6552099431345591756L;

		@Override
		public boolean canImport(TransferSupport support) {

			if (!support.isDrop()) {
				return false;
			}

			// we only import Strings
			if (	support.isDataFlavorSupported(DataFlavor.stringFlavor) ||
					support.isDataFlavorSupported(DataFlavor.javaFileListFlavor) ||
					support.isDataFlavorSupported(_URIListDataflavorString) ||
					support.isDataFlavorSupported(_URIListDataflavorCharArray))
			{
				// check if the source actions (a bitwise-OR of supported actions)
				// contains the COPY action
				boolean copySupported = (COPY & support.getSourceDropActions()) == COPY;
				if (copySupported) {
					support.setDropAction(COPY);
					return true;
				}
			}

			// Reject transfer
			return false;
		}
		
		@Override
		public boolean importData(TransferSupport support)
		{
			if (!canImport(support) || DisplayController.getCurrentFrame() == null) return false;

			// Get the location of where to drop the import
			DropLocation location = support.getDropLocation();

			// Convert it into expeditee space
			Point expediteeDropPoint = SwingConversions.fromSwingPoint(location.getDropPoint());

			try {

				// The list of data flavors are ordered by most rich to least ..
				// keep trying until first
				// data flavor recognized.
				DataFlavor[] flavors = support.getTransferable().getTransferDataFlavors();
				for (DataFlavor df : flavors) {

					System.out.println(df);

					if (df == DataFlavor.stringFlavor) { // import as text item

						String str = (String) support.getTransferable().getTransferData(DataFlavor.stringFlavor);

						if (str != null && str.length() > 0) {
							importString(str, expediteeDropPoint, false);
							return true;
						}

					// Usually Windows and MAC enviroments
					// Windows has other random types...
					} else if (df == DataFlavor.javaFileListFlavor || df.getSubType().equals("x-java-file-list")) {

						List<? extends File> files = (List<? extends File>) support.getTransferable().getTransferData(DataFlavor.javaFileListFlavor);

						importFileList(files, expediteeDropPoint, false);

						return true;

					// Usually GNOME and KDE enviroments
					} else if (df.equals(_URIListDataflavorString)) {

						String data = (String) support.getTransferable().getTransferData(_URIListDataflavorString);

						List<File> files = textURIListToFileList(data);

						importFileList(files, expediteeDropPoint, false);

						return true;

					} else if (df.equals(_URIListDataflavorCharArray)) {

						char[] data = (char[]) support.getTransferable().getTransferData(_URIListDataflavorCharArray);

						String uriString = new String(data);

						List<File> files = textURIListToFileList(uriString);

						importFileList(files, expediteeDropPoint, false);

						return true;
					}
				}

			} catch (UnsupportedFlavorException e) {
				MessageBay.displayMessage("Drag and drop for that type of data is not supported");
			} catch (IOException e) {
				e.printStackTrace();
				MessageBay.displayMessage("Failed to import data in Expeditee");
			}

			return false;
		}
		
	}

}
